/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * private.h
 * Private definitions, data structures, macros and functions for libespnet
 * Copyright (C) 2005-2006 Simon Newton
 */

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <espnet.h> 
#include <espnet/packets.h> 

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>


#ifndef _defs_defined
#define _defs_defined


#define ESPNET_PORT 3333		// port to bind to

#define FIRMWARE_VER 0x01

#define TRUE 1
#define FALSE 0

#define RECV_NO_DATA 1

// non espnet specific
#define SA struct sockaddr
#define SI struct in_addr

// Macros

#define min(a,b)    ((a) < (b) ? (a) : (b))
#define max(a,b)    ((a) > (b) ? (a) : (b))

// byte ordering macros
#define bswap_16(x)	((((x) >> 8) & 0xff) | (((x) & 0xff) << 8))

//#define bytes_to_short(x,y) ( (x << 8) & 0xff00 | (l & 0x00FF) ) ;

// htols : convert short from host to little endian order
// ltohs : convert short from little endian to host order
#ifdef HAVE_ENDIAN_H
# if BYTE_ORDER == __BIG_ENDIAN
#  define htols(x) 	bswap_16 (x)
# else
#  define htols(x)	(x)
# endif
#else
# if BYTE_ORDER == BIG_ENDIAN
#  define htols(x) 	bswap_16 (x)
# else
#  define htols(x)	(x)
# endif
#endif


#define espnet_error_malloc() espnet_error("%s : malloc error %s" , __FUNCTION__, strerror(errno) ) 
#define espnet_error_realloc() espnet_error("%s : realloc error %s" , __FUNCTION__, strerror(errno) ) 
#define espnet_error_nullnode() espnet_error("%s : argument 1 (espnet_node) was null" , __FUNCTION__ ) 
										
// check if the node is null and return an error
#define return_if_null(node) if (node == NULL) { \
	espnet_error_nullnode() ; \
	return ESPNET_EARG ; \
}

// return if the node is off
#define return_if_off(node) if (node->state.mode == ESPNET_OFF) { \
	espnet_error("%s : node is not on" , __FUNCTION__ ) ; \
	return ESPNET_ESTATE ; \
}
										
// return if the node is on
#define return_if_on(node) if (node->state.mode == ESPNET_ON) { \
	espnet_error("%s : node is not on" , __FUNCTION__ ) ; \
	return ESPNET_ESTATE ; \
}


// Esp Poll Reply 
//-----------------------------------------------------------------------------

// type in espnet.h

typedef enum {
	ESPNET_REPLY_OPTION_CONFIGURED = 0x00,
	ESPNET_REPLY_OPTION_DHCP = 0x01,
	ESPNET_REPLY_OPTION_ERRCOR = 0x02
} espnet_reply_option;
			
// Esp Poll
//-----------------------------------------------------------------------------

// type in espnet.h


// Esp Ack
//-----------------------------------------------------------------------------

typedef enum {
	ESPNET_ACK_STATUS_OK = 0x00,
	ESPNET_ACK_STATUS_CRC_FAILED = 0x01,
	ESPNET_ACK_STATUS_CRC_UNSUPPORTED = 0xFF,
} espnet_ack_status;


// Esp dmx data
//-----------------------------------------------------------------------------

typedef enum  {
	ESPNET_DMX_TYPE_DATA = 0x01,
	ESPNET_DMX_TYPE_PAIRS = 0x02,
	ESPNET_DMX_TYPE_RLE = 0x04,
} espnet_dmx_type ;


// yet to do config crap

typedef enum  {
	ESPNET_OUT_BEHAV_BCAST = 0x01,
	ESPNET_OUT_BEHAV_PEER = 0x02,
	ESPNET_OUT_BEHAV_CBCAST = 0x04,
	ESPNET_OUT_BEHAV_ANY = 0x08,
} espnet_output_behaviour ;

typedef enum  {
	ESPNET_IN_BEHAV_BCAST = 0x01,
	ESPNET_IN_BEHAV_PEER = 0x02,
	ESPNET_IN_BEHAV_CBCAST = 0x04,
	ESPNET_IN_BEHAV_RLE = 0x08,
} espnet_input_behaviour ;


// End packet enums
//-----------------------------------------------------------------------------


/*
 * Data structures to manage callbacks
 */

// packet callbacks have a function and some user data
typedef struct {
	int (*fh)(espnet_node n, void *p, void *data) ;
	void *data;	
} callback_t ;

/*
 * the dmx callback is triggered when a dmx packet arrives
 */
typedef struct {
	int (*fh)(espnet_node n, uint8_t universe, int length, uint8_t *data, void *d) ;
	void *data;	
} dmx_callback_t ;



// struct to store callbacks
typedef struct {
	callback_t recv;
	callback_t send;
	callback_t poll;
	callback_t reply;
	callback_t dmx;
	callback_t ack;	
	dmx_callback_t dmx_c ;
} node_callbacks_t ;


// End callback structures
//-----------------------------------------------------------------------------


// the status of the node
typedef enum {
	ESPNET_OFF,
	ESPNET_STANDBY,
	ESPNET_ON
} node_status_t ;


// struct to hold the state of the node
typedef struct {
	node_status_t mode;
	SI ip_addr;
	SI bcast_addr ;
	uint8_t hw_addr[ESPNET_MAC_LENGTH] ;	
	char name[ESPNET_NAME_LENGTH] ;
	int verbose ;
	espnet_node_type type;	// input, output ... ???
	uint8_t strtcd;
	uint8_t ttl;
	uint8_t tos;
	uint8_t	uni;
	uint8_t option;		
	uint8_t bhr ;		// behaviour
	SI peer_ip;				// peer ip
} node_state_t ;


/**
 * The main node structure
 */
typedef struct espnet_node_s{
	int sd ;									// the two sockets
	node_state_t state;							// the state struct
	node_callbacks_t callbacks;					// the callbacks struct
} espnet_node_t ;


typedef espnet_node_t *node ;

// Global variables
extern char *espnet_errstr ;

// Function definitions follow
 
// espnet.c
void espnet_error(const char *fmt, ...) ;

// exported from receive.c
int espnet_handle(node n, espnet_packet p ) ;
int16_t espnet_get_type(espnet_packet p ) ;

// exported from transmit.c
int espnet_tx_poll(node n, const char *ip, espnet_poll_request type) ;
int espnet_tx_ack(node n, SI dst, uint8_t status, uint8_t crc) ;
int espnet_tx_poll_reply(node n, SI dst) ;
int espnet_tx_dmx(node n, uint8_t universe, int length, uint8_t *data) ;


// exported from network.c
ssize_t espnet_net_recv(node n, espnet_packet p, int block) ;
int espnet_net_send(node n, espnet_packet p) ;
int espnet_net_init(node n, const char *ip) ;
int espnet_net_start(node n) ;
int espnet_net_close(node n) ;


#endif
