/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * receive.c
 * Handles the receiving of datagrams
 * Copyright (C) 2005 Simon Newton
 */


#include "private.h"


/** checks if the callback is defined, if so call it passing the packet and 
 * the user supplied data. 
 * If the callbacks return a non-zero result, further processing is canceled.
 */
int espnet_check_callback(node n, espnet_packet p, callback_t callback) {
	if(callback.fh != NULL) 
		return callback.fh(n,p, callback.data) ;

	return 0;

}

/**
 * Handle an esp poll packet
 */
int espnet_handle_poll(node n, espnet_packet p) {

	// run callback if defined
	if(espnet_check_callback(n, p, n->callbacks.poll) )
		return ESPNET_EOK ;

	if(p->data.poll.type == ESPNET_POLL_REQUEST_ACK) { 
		// we need to do a crc check here
		espnet_tx_ack(n, p->from, ESPNET_ACK_STATUS_OK , 0) ;
	} else {
		espnet_tx_poll_reply(n, p->from) ;
	}
	return ESPNET_EOK;
}


/**
 * handle an esp poll reply
 */
int espnet_handle_reply(node n, espnet_packet p) {

	// run callback if defined
	if(espnet_check_callback(n, p, n->callbacks.reply) )
		return ESPNET_EOK;

	return ESPNET_EOK;
}



/**
 * handle an esp ack
 */
int espnet_handle_ack(node n, espnet_packet p) {

	// run callback if defined
	if(espnet_check_callback(n, p, n->callbacks.ack) )
		return ESPNET_EOK;

	return ESPNET_EOK;
}


/**
 * handle a art dmx packet
 *
 *
 */
int espnet_handle_dmx(node n, espnet_packet p) {
	uint8_t tmp[ESPNET_DMX_LENGTH] ;
	int i,j, lim ;
	uint8_t *val ;
	
	// run callback if defined	
	if(espnet_check_callback(n, p, n->callbacks.dmx) )
		return ESPNET_EOK;

	if(p->data.dmx.type == ESPNET_DMX_TYPE_DATA) {
		if(n->callbacks.dmx_c.fh != NULL) 
			n->callbacks.dmx_c.fh(n, 
								  p->data.dmx.universe,
								  ntohs(p->data.dmx.size),
								  p->data.dmx.data,
								  n->callbacks.dmx_c.data) ;
	} else if (p->data.dmx.type == ESPNET_DMX_TYPE_PAIRS) {
		// not implemented, we'd have to buffer the data for each universe
		// or maybe we just send this up and they can deal with it

	} else if (p->data.dmx.type == ESPNET_DMX_TYPE_RLE) {
		// decode RLE

		i=0 ;
		val = p->data.dmx.data ;
		while(i < ESPNET_DMX_LENGTH && val != p->data.dmx.data + ntohs(p->data.dmx.size)) {

			if( *val == 0xFD) {
				//escape char
				val++ ;
				tmp[i] = *val ;
			} else if( *val == 0xFE) {
				// encoded
				val++ ;
				lim = *val ;
				for(j=0; j < lim; j++  ) {
					tmp[i+j] = *val ;
				}
			} else {
				tmp[i] = *val ;
			}
			val++ ;
			i++ ;
		}

		if(n->callbacks.dmx_c.fh != NULL) 
			n->callbacks.dmx_c.fh(n, 
								  p->data.dmx.universe,
								  ntohs(p->data.dmx.size),
								  p->data.dmx.data,
								  n->callbacks.dmx_c.data) ;

	} else {
		printf("Data type not recognised %hhx\n",  p->data.dmx.type) ;
	}
	
	return ESPNET_EOK;
}




/**
 * the main handler for an espnet packet. calls 
 * the appropriate handler function
 *
 */
int espnet_handle(node n, espnet_packet p) {

	if(espnet_check_callback(n, p, n->callbacks.recv) )
		return ESPNET_EOK;

	switch(p->type) {
		case ESPNET_POLL:
			espnet_handle_poll(n, p) ;
			break;
		case ESPNET_REPLY:
			espnet_handle_reply(n,p) ;
			break;
		case ESPNET_ACK:
			espnet_handle_ack(n,p) ;
			break;
		case ESPNET_DMX:
			espnet_handle_dmx(n, p) ;
			break;
		case -1:
			printf("not an EspNet packet\n") ;
			return -1 ;
			break;
		default:
			printf("espnet but not yet implemented!, op was %hx\n", p->type) ;
	}
	return ESPNET_EOK  ; 

}

/**
 * this gets the opcode from a packet
 */
int16_t espnet_get_type(espnet_packet p) {

	if(! memcmp( &p->data, "ES", 2)) {
		p->type =  ntohl(p->data.dmx.head)  ;
		
		return p->type ;
	} else {
		return 0 ;
	}
}




